-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: debug.vhd
-- *
-- * Version: V3.0
-- *
-- * Release Date:
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: SimpleCPU simulation disassembler
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *
-- =============================================================================================================



LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY debug IS
PORT (
  CLK  : IN STD_LOGIC ;
  CLR  : IN STD_LOGIC ;
  ADDR : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
  DATA : IN STD_LOGIC_VECTOR(15 DOWNTO 0)  );
END debug;

ARCHITECTURE debug_arch OF debug IS

  TYPE state_type IS (fetch, decode, execute);
	 
  TYPE opcode_type IS (MOVE, ADD, SUB, BW_AND,
                       LOAD, STORE, ADDM, SUBM,
                       JUMP, JUMP_Z, JUMP_NZ, JUMP_C,
							  ROTL, ROTR, STOP,
                       NU, XX );	
	 
  SIGNAL opcode : opcode_type;
  SIGNAL imm8  : STD_LOGIC_VECTOR(7 downto 0);	
  SIGNAL phase  : state_type ;

BEGIN

  simulation: PROCESS (clk, clr)

    VARIABLE present_state: state_type;
	 
  BEGIN
    IF clr = '1'
    THEN 
      present_state := fetch;
    ELSIF clk='1' and clk'event
    THEN  
	   phase <= present_state;
      CASE present_state IS
        WHEN fetch => 
		    present_state := decode;
          CASE DATA(15 downto 14) IS
            WHEN "00" =>
              CASE DATA(13 downto 12) IS
                WHEN "00" => 
                  opcode <= MOVE;
                  imm8 <= DATA(7 downto 0);
                WHEN "01" => 
                  opcode <= ADD;
                  imm8 <= DATA(7 downto 0);
                WHEN "10" => 
                  opcode <= SUB;
                  imm8 <= DATA(7 downto 0);
                WHEN "11" => 
                  opcode <= BW_AND;
                  imm8 <= DATA(7 downto 0);
					 WHEN OTHERS => 
                  opcode <= XX;
                  imm8 <= "XXXXXXXX";
              END CASE;						
            WHEN "01" =>
              CASE DATA(13 downto 12) IS
                WHEN "00" => 
                  opcode <= LOAD;
                  imm8 <= DATA(7 downto 0);
                WHEN "01" => 
                  opcode <= STORE;
                  imm8 <= DATA(7 downto 0);
                WHEN "10" => 
                  opcode <= ADDM;
                  imm8 <= DATA(7 downto 0);
                WHEN "11" => 
                  opcode <= SUBM;
                  imm8 <= DATA(7 downto 0);
                WHEN OTHERS => 
                  opcode <= XX;
                  imm8 <= "XXXXXXXX";
				  END CASE;
            WHEN "10"   =>
              CASE DATA(13 downto 12) IS
                WHEN "00" => 
                  opcode <= JUMP;
                  imm8 <= DATA(7 downto 0);
                WHEN "01" => 
                  opcode <= JUMP_Z;
                  imm8 <= DATA(7 downto 0);
                WHEN "10" => 
                  opcode <= JUMP_NZ;
                  imm8 <= DATA(7 downto 0);						
                WHEN "11" => 
                  opcode <= JUMP_C;
                  imm8 <= DATA(7 downto 0);						
                WHEN OTHERS => 
                  opcode <= XX;
                  imm8 <= "XXXXXXXX";	
              END CASE;	
            WHEN "11"   =>
				  CASE DATA(13 downto 12) IS
                WHEN "00" => 
                  opcode <= ROTR;
                  imm8 <= "UUUUUUUU";
                WHEN "01" => 
                  opcode <= ROTL;
                  imm8 <= "UUUUUUUU";				
                WHEN "11" => 
                  opcode <= STOP;
                  imm8 <= "UUUUUUUU";				
						
                WHEN OTHERS => 		
                  opcode <= XX;
                  imm8 <= "XXXXXXXX";	
				  END CASE;		
            WHEN OTHERS => 
              opcode <= XX;
              imm8 <= "XXXXXXXX";	
				END CASE;
        WHEN decode => 
          present_state := execute;
		  WHEN execute => 
          present_state := fetch;
        WHEN OTHERS =>
          null;
      END CASE;		  
    END IF;
  END PROCESS simulation;

END debug_arch;

  
